#!/usr/bin/perl -w
=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2002, Guardian Digital Inc.
 | $Id: sysreport.pl,v 1.4.2.1 2003/04/18 12:43:33 rwm Exp $
 |
 | Ryan W. Maple <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  # use perl;
  use strict;
  use Getopt::Std qw(getopts);

  #
  # Start your engines (bewm).
  #
  my %OPTS; getopts('sh', \%OPTS);
  my $LOGDIR = '/var/log/reports';
  my $FMT = undef;
  my $FOUND = undef;
  my $UPTIME = &get_uptime();
  my $LOADAVG = &get_uptime(1);
  my $CURRENT_DATE = `date`; chomp $CURRENT_DATE;
  my $KERNELVER = `uname -r`; chomp $KERNELVER;
  my $FILE = "$LOGDIR/sysreport-"; $FILE .= `date +%Y.%m.%d`; chomp $FILE;


  #
  # If we were NOT given the -s option then save the report to a file and
  # (later on) email it to the system administrator.
  #
  &Usage() if ($OPTS{'h'});

  if (! $OPTS{'s'}) {
    close STDOUT; open STDOUT, "> $FILE" or die "Could write to $FILE: $!";
  }


  #
  # System Information
  #
  &Header();
  $FMT = "  %-15.15s %-54.54s\n";

  print "\n";
  printf($FMT, 'Report Time:', $CURRENT_DATE);
  print "\n";
  printf($FMT, 'System Uptime:', $UPTIME);
  printf($FMT, 'Load Average:', $LOADAVG);
  printf($FMT, 'Kernel Version:', $KERNELVER);
  print "\n";


  #
  # Network Device Information
  #
  print "\#\n\# Network Device Information\n\#\n\n";
  $FMT = "  %-10.10s  %-18.18s  %10.10s %4.4s  %10.10s %4.4s\n";

  printf($FMT, 'Interface', 'Address/Mask', 'RX Packets', '%err', 'TX Packets', '%err');
  print "  ", '-'x64, "\n";

  foreach my $IF (&ParseIfconfig()) {
    if ($IF->{'RX'} && $IF->{'TX'}) {
      printf($FMT, $IF->{'name'}, "$IF->{'addr'}/$IF->{'mask'}",
                   $IF->{'RX'}, $IF->{'RX_e'}, $IF->{'TX'}, $IF->{'TX_e'});
    }
    else {
      printf($FMT, $IF->{'name'}, "$IF->{'addr'}/$IF->{'mask'}", 'n/a', '', 'n/a', '');
    }
  }


  #
  # Network Services Information
  #
  print "\n\#\n\# Network Services Information\n\#\n\n";
  $FMT = "  %-8.8s  %15.15s  %5.5s  %10.10s%16.16s\n";

  printf($FMT, 'Protocol', 'Bound To', 'Port', 'Service', 'Process');
  print "  ", '-'x60, "\n";

  foreach my $NET (sort { $a->{'lport'} <=> $b->{'lport'} } &ParseNetstat()) {
    my $TMP = getservbyport($NET->{'lport'}, $NET->{'proto'});
    $TMP = '<unknown>' if (! $TMP);

    printf($FMT, $NET->{'proto'}, $NET->{'lip'}, $NET->{'lport'}, $TMP,
                 $NET->{'service'});
  }


  #
  # Filesystem Information
  #
  print "\n\#\n\# Filesystem Information\n\#\n\n";
  $FMT = "  %-12.12s  %-8.8s %-18.18s  %6.6s  %5.5s  %5.5s\n";

  printf($FMT, 'Filesystem', 'Type', 'Mount Point', 'Size', '%Used', 'Avail');
  print "  ", '-'x63, "\n";

  foreach my $FS (&ParseDf()) {
    printf($FMT, $FS->{'fs'}, $FS->{'type'}, $FS->{'mount'},
                 $FS->{'size'}, $FS->{'used'}, $FS->{'avail'});
  }


  #
  # Memory Usage
  #
  print "\n\#\n\# Memory Usage\n\#\n\n";
  $FMT = "  %-7.7s  %5.5s  %5.5s  %6.6s\n";

  printf($FMT, 'Type', 'Total', '%Used', 'Free');
  print "  ", '-'x29, "\n";

  foreach my $MEM (&ParseFree()) {
    my $TMP = sprintf("%0.0f", (($MEM->{'used'} / $MEM->{'total'}) * 100));
    printf($FMT, $MEM->{'type'}, "$MEM->{'total'}MB", "$TMP%", "$MEM->{'free'}MB",
                 "$MEM->{'used'}MB ($TMP%)");
  }

  #
  # Latest Packages
  #
  print "\n\#\n\# Most Recent Packages Installed/Upgraded\n\#\n\n";
  $FMT = "  %-20.20s  %9.9s  %9.9s   %-24.24s\n";

  printf($FMT, 'Package', 'Version', 'Release', 'Installed');
  print "  ", '-'x69, "\n";

  foreach my $RPM (&ParseRPMLast()) {
    printf($FMT, $RPM->{'pack'}, $RPM->{'ver'}, $RPM->{'rel'}, $RPM->{'date'});
  }

  #
  # Mail Queue
  #
  print "\n\#\n\# Mail Queue\n\#\n\n";
  $FMT = "  %-10.10s  %-19.19s  %-39.39s\n";

  printf($FMT, 'Queue ID', 'Arrival Time', 'Sender');
  print "  ", '-'x72, "\n";

  $FOUND = 0;
  foreach my $Q (&ParseMailQueue()) {
    printf($FMT, $Q->{'id'}, $Q->{'arrival'}, $Q->{'sender'});
    $FOUND++;
  }
  if (! $FOUND) { print "  The Mail Queue is Empty\n"; }


  #
  # DNZ Zones
  #
  print "\n\#\n\# DNS Zones\n\#\n\n";
  $FMT = "  %-6.6s  %-64.64s\n";

  printf($FMT, 'Type', 'Zone');
  print "  ", '-'x72, "\n";

  $FOUND = 0;
  foreach my $ZONE (&ParseDNSZones()) {
    next if (! $ZONE);
    printf($FMT, $ZONE->{'type'}, $ZONE->{'zone'});
    $FOUND++;
  }
  if (! $FOUND) { print "  No Zones are Configured\n"; }


  #
  # Mail Domains
  #
  print "\n\#\n\# Mail Domains\n\#\n\n";
  $FMT = "  %-20.20s  %-15.15s  %-11.11s\n";

  printf($FMT, 'Domain', 'Postmaster', '# Addresses');
  print "  ", '-'x50, "\n";

  $FOUND = 0;
  foreach my $VIRT (&ParsePostfixVirtual()) {
    next if (! $VIRT);
    printf($FMT, $VIRT->{'domain'}, $VIRT->{'postmaster'}, $VIRT->{'cnt'});
    $FOUND++;
  }
  if (! $FOUND) { print "  No Mail Domains are Configured\n"; }


  #
  # Virtual Hosts
  #
  print "\n\#\n\# Virtual Hosts\n\#\n\n";
  $FMT = "  %-45.45s  %-15.15s  %8.8s\n";

  printf($FMT, 'Website', 'IP Address', '# Errors');
  print "  ", '-'x72, "\n";

  $FOUND = 0;
  foreach my $VIRT (sort { $a->{'site'} cmp $b->{'site'} } &ParseHttpdConf()) {
    next if (! $VIRT);
    printf($FMT, $VIRT->{'site'}, $VIRT->{'addr'}, $VIRT->{'cnt'});
    $FOUND++;
  }
  if (! $FOUND) { print "  No Virtual Hosts are Configured\n"; }


  #
  # Potentially email report to administrator.
  #
  if (! $OPTS{'s'}) {
    open H, '/etc/hostname'; my $H = <H>; chomp $H; close H;
    my $DATE = `date "+%a %b %e, %Y"`; chomp $DATE;
    my $TO = undef;

    open ADDRESS, '/etc/logsum.conf';
      while (my $LINE = <ADDRESS>) {
        if ($LINE =~ m/^EMAIL\s+\=\s+(.*)$/) { $TO = $1; }
      }
    close ADDRESS;

    if ($TO) {
      open MAIL, '| /usr/sbin/sendmail -t';
        print MAIL "From: reporting\@$H\n";
        print MAIL "Subject: [$DATE] System Report for '$H'\n";
        print MAIL "To: $TO\n";
        print MAIL "X-EnGarde-sysreport: $H\n\n";

        open REPORT, $FILE;
          while (<REPORT>) { print MAIL $_; }
        close REPORT;
      close MAIL;
    }
  }


  #
  # Destruction
  #
  &Footer();
  exit 0;

################################################################################
sub ParseHttpdConf() {
  my @RV = ();
  my $TMP;
  my $IN_VHOST = 0;

  open HTTPD, '/etc/httpd/conf/httpd.conf' or return undef;
    while (my $LINE = <HTTPD>) {
      chomp $LINE;

      if ($LINE =~ m/\s?<VirtualHost\ ([0-9\.]+)\:([0-9]+)>$/) {
        $TMP->{'addr'} = $1;
        $TMP->{'port'} = $2;
        $TMP->{'cnt'}  = 0;
        $IN_VHOST = 1;
      }

      elsif ($IN_VHOST && $LINE =~ m/\s?ServerName\ (.*)$/) {
        if ($TMP->{'port'} == 80)     { $TMP->{'site'} = "http://$1/";  }
        elsif ($TMP->{'port'} == 443) { $TMP->{'site'} = "https://$1/"; }
      }

      elsif ($IN_VHOST && $LINE =~ m/\s?ErrorLog\ ([^\s]+)$/) {
        open LOG, $1; while (<LOG>) { $TMP->{'cnt'}++; } close LOG;
      }

      elsif ($IN_VHOST && $LINE =~ m/\s?<\/VirtualHost>$/) {
        push @RV, $TMP;
        $IN_VHOST = 0;
        $TMP = undef;
      }
    }
  close HTTPD;

  return @RV;
}

################################################################################
sub ParsePostfixVirtual() {
  my @RV = ();
  my $TMP;
  my $IN_DOMAIN = 0;

  open VIRTUAL, '/etc/postfix/virtual' or return undef;
    while (my $LINE = <VIRTUAL>) {
      chomp $LINE;

      if ($LINE =~ m/^([^\s]+)\s+VIRTUALDOMAIN$/) {
        $IN_DOMAIN = 1;
        $TMP->{'domain'} = $1;
        $TMP->{'cnt'} = 0;
      }

      elsif ($IN_DOMAIN && $LINE =~ m/^\@$TMP->{'domain'}\s+(.*)$/) {
        $TMP->{'postmaster'} = $1;
      }

      elsif ($IN_DOMAIN && $LINE eq '') {
        push @RV, $TMP;
        $TMP = undef;
        $IN_DOMAIN = 0;
      }

      elsif ($IN_DOMAIN) { $TMP->{'cnt'}++; }
    }
  close VIRTUAL;

  return @RV;
}

################################################################################
sub ParseDNSZones() {
  my @RV = ();
  my $TMP;
  my $IN_ZONE = 0;

  open NAMED, '/var/chroot/named/etc/named.conf' or return undef;
    while (my $LINE = <NAMED>) {
      chomp $LINE;

      if ($LINE =~ m/^zone\ \"([^\"]+)\"\ \{$/) {
        $TMP->{'zone'} = $1;
        $IN_ZONE = 1;
      }

      elsif ($IN_ZONE && $LINE =~ m/^\s+type\ (.*)$/) {
        if ($1 eq 'master;')   { $TMP->{'type'} = 'Master'; }
        elsif ($1 eq 'slave;') { $TMP->{'type'} = 'Slave'; }
        else                  { $TMP->{'type'} = 'Other'; }
      }

      elsif ($IN_ZONE && $LINE eq '};') {
        push @RV, $TMP if (($TMP->{'zone'} ne 'localhost') &&
                           ($TMP->{'zone'} ne '0.0.127.in-addr.arpa') &&
                           ($TMP->{'zone'} ne '.'));
       $IN_ZONE = 0; $TMP = undef;
      }
    }
  close NAMED;

  return @RV;
}

################################################################################
sub ParseMailQueue() {
  my @RV = ();
  my $TMP;

  open MAILQ, 'mailq 2>/dev/null |';
    while (my $LINE = <MAILQ>) {
      next if ($LINE !~ m/^[A-Z0-9]+\s+/);
      chomp $LINE;

      $TMP->{'id'} = substr $LINE, 0, 10;
      $TMP->{'arrival'} = substr $LINE, 20, 20;
      $TMP->{'sender'} = substr $LINE, 41;

      push @RV, $TMP; $TMP = undef;
    }
  close MAILQ;

  return @RV;
}

################################################################################
sub ParseNetstat() {
  my @RV = ();

  open NETSTAT, 'netstat --inet -anp 2>/dev/null|';
    while (my $LINE = <NETSTAT>) {
      next if ($LINE !~ m/^[tu]/);
      chomp $LINE;

      my @TMP = split /\s+/, $LINE;
      my @TMP2 = split /\:/, $TMP[3];
      $TMP[6] = "" if (! $TMP[6]);

      next if ($TMP[4] ne "0.0.0.0:*");

      push @RV, { 'proto' => $TMP[0], 'lip' => $TMP2[0], 'lport' => $TMP2[1],
                  'service' => $TMP[6] };
    }
  close NETSTAT;

  return @RV;
}

################################################################################
sub ParseRPMLast() {
  my @RV = ();
  my $COUNT = 0;

  open RPM, "rpm -qa --qf '%|INSTALLTIME?{%{INSTALLTIME}}:{000000000}| %{NAME}\t%{VERSION}\t%{RELEASE}\t%|INSTALLTIME?{%-27{INSTALLTIME:date}}:{(not installed)}|\n' | sort -nr |";
    while (my $LINE = <RPM>) {
      chomp $LINE;

      my @TMP = split /\t/, $LINE;
      $TMP[0] =~ s/^[0-9]+\ //g;

      push @RV, { 'pack' => $TMP[0], 'ver' => $TMP[1], 'rel' => $TMP[2],
                  'date' => $TMP[3] };

      $COUNT++;
      goto outrpm if ($COUNT >= 10);
    }
outrpm:
  close RPM;

  return @RV;
}

################################################################################
sub ParseFree() {
  my @RV = ();

  open FREE, 'free -m |';
    while (my $LINE = <FREE>) {
      next if $LINE !~ /^[MS]/;
      chomp $LINE;

      my @TMP = split /\s+/, $LINE;

      if ($TMP[0] eq 'Mem:') {
        push @RV, { 'type' => 'RAM',   'total' => $TMP[1],
                    'used' => $TMP[2], 'free'  => $TMP[3] };
      }
      elsif ($TMP[0] eq 'Swap:') {
        push @RV, { 'type' => 'Swap',  'total' => $TMP[1],
                    'used' => $TMP[2], 'free'  => $TMP[3] };
      }
    }
  close FREE;

  return @RV;
}

################################################################################
sub ParseDf() {
  my @RV = ();

  open DF, '/bin/df -hPT |';
    while (my $LINE = <DF>) {
      next if $LINE !~ /^\//;
      chomp $LINE;

      my @TMP = split /\s+/, $LINE;

      push @RV, { 'fs' => $TMP[0], 'mount' => $TMP[6], 'size' => $TMP[2],
                  'avail' => $TMP[4], 'used' => $TMP[5], 'type' => $TMP[1] };
    }
  close DF;

  return @RV;
}

################################################################################
sub ParseIfconfig() {
  my @RV = ();
  my $IN_BLOCK = 0;
  my $TMP;

  # Do the deed.
  open IFCONFIG, '/sbin/ifconfig |';
    while (my $LINE = <IFCONFIG>) {
      chomp $LINE;

      # IF Name
      if ($LINE =~ m/^([a-z0-9\:]+)\s+/) { $TMP->{'name'} = $1; $IN_BLOCK = 1; }

      # IP Address and Netmask.
      elsif ($IN_BLOCK && $LINE =~ m/^\s+inet\ addr\:([^\ ]+).*Mask\:(.*)$/) {
        $TMP->{'addr'} = $1;

        my $MASK = shift;
        my @BYTES = split /\./, $2;
        my $CIDR = 0;

        for (@BYTES) {
          my $BITS = unpack( "B*", pack( "C", $_ ) );
          $CIDR += $BITS =~ tr /1/1/;
        }

        $TMP->{'mask'} = $CIDR;
      }

      elsif ($IN_BLOCK && $LINE =~ m/^\s+RX\ packets:([0-9]+)\ errors\:([0-9]+)/) {
        if (! $1 && ! $2) {
          $TMP->{'RX'} = $TMP->{'RX_e'} = 0;
        } else {
          $TMP->{'RX'} = $1; $TMP->{'RX_e'} = sprintf("%0.0f%%", (($2 / $1) * 100));
        }
      }
      elsif ($IN_BLOCK && $LINE =~ m/^\s+TX\ packets:([0-9]+)\ errors\:([0-9]+)/) {
        if (! $1 && ! $2) {
          $TMP->{'TX'} = $TMP->{'TX_e'} = 0;
        } else {
          $TMP->{'TX'} = $1; $TMP->{'TX_e'} = sprintf("%0.0f%%", (($2 / $1) * 100));
        }
      }

      # End of block, get ready for next.
      elsif ($LINE eq '') { push @RV, $TMP; $IN_BLOCK = 0; $TMP = undef; }
    }
  close IFCONFIG;

  return @RV;
}

################################################################################
sub get_uptime() {
  my $FCN = shift;
  my $U = `uptime`;

  if ($FCN) { ($U) = ($U =~ m/load\ average\:\ (.*)$/); }
  else {
    if ($U =~ /days/)   { ($U) = ($U =~ m/^.*up\ (.*\:[0-9]{2}),/);   }
    elsif ($U =~ /min/) { ($U) = ($U =~ m/^.*up\ (.*[0-9]+\ min),/);  }
    else                { ($U) = ($U =~ m/^.*up\ +(.*\:[0-9]{2}),/); }
  }

  return $U;
}

################################################################################
sub Header() {
  # Get our hostname and EnGarde version.
  open H, '/etc/hostname'; my $H = <H>; chomp $H; close H;

  # Ereet header.
  print '+', '-'x72, "+\n";
  print "| EnGarde Secure Linux                   Community Edition Sample Report |\n";
  printf("| %-70.70s |\n", '');
  printf("| %-70.70s |\n", $H);
  print '+', '-'x72, "+\n";
}

################################################################################
sub Footer() {
  print "\n";
}

################################################################################
sub Usage() {
  print STDOUT "Usage: $0 { -s }\n";
  print STDOUT "  -s	Do not save or email report\n\n";
  print STDOUT "Report bugs to <ryan\@guardiandigital.com>.\n";
  exit -1;
}
